//
// TinyPTC with DirectDraw
//
//
#include "main.h"
#include "tinyptc.h"
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <windows.h>
#include "resource.h"

#ifdef MINIFMOD
// minifmod include & lib
#include "minifmod.h"
#pragma comment(lib,"minifmod.lib")
#pragma comment(lib,"winmm.lib")
#pragma comment(linker, "/nodefaultlib:libc.lib")
//--- Module
#include "TAUSTI.H"

//-------------------------- MINIFMOD -------------------------*
// module struct
typedef struct 
{
	int length;
	int pos;
	void *data;
} MEMFILE;

FMUSIC_MODULE *pModule;

unsigned int memopen(char *name)
{
	MEMFILE *memfile;

	memfile = (MEMFILE *)calloc(sizeof(MEMFILE),1);

	memfile->length = sizeof(music);
	memfile->data = malloc(memfile->length);
	memfile->pos = 0;
	memcpy(memfile->data,music,memfile->length);

	return (unsigned int)memfile;
}

void memclose(unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;
	free(memfile->data);
}

int memread(void *buffer, int size, unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	if (memfile->pos + size >= memfile->length)
		size = memfile->length - memfile->pos;

	memcpy(buffer, (char *)memfile->data+memfile->pos, size);
	memfile->pos += size;
	
	return size;
}

void memseek(unsigned int handle, int pos, signed char mode)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	if (mode == SEEK_SET) 
		memfile->pos = pos;
	else if (mode == SEEK_CUR) 
		memfile->pos += pos;
	else if (mode == SEEK_END)
		memfile->pos = memfile->length + pos;

	if (memfile->pos > memfile->length)
		memfile->pos = memfile->length;
}

int memtell(unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	return memfile->pos;
}
#endif // ifdef minifmod

//------------------------------------------------------------
// Setup screen
#define WIDTH 640
#define HEIGHT 480
#define SIZE WIDTH*HEIGHT

// Setup a buffer for our reading and writing (FrameBuffer)
static int buffer[SIZE];

// Function declares
void Cls(int rgb);
void WritePixel(int x,int y,int rgb);
void edge(int x1,int y1,int x2,int y2,unsigned int color);
//
void stopzic();
void initfx();
void render_effect();

//-----------------------------------------------------------------
// Our Functions
//-----------------------------------------------------------------
#ifdef STARS
	#include "FX/Starfields.h"
#endif
#ifdef STARSP1
	#include "FX/Starfields.h"
#endif
#ifdef STARSP3
	#include "FX/Starfields.h"
#endif
//----------------
#ifdef TUNNEL
	#include "FX/DotsTunnel.h"
#endif
#ifdef RASTERL
	#include "FX/Rasters.h"
#endif
#ifdef RASTERROLL
	#include "FX/Rasters.h"
#endif
#ifdef RASTERROLL2
	#include "FX/Rasters.h"
#endif
#include "FX/Scrolls.h"
//-----------------------------------------------
//-                   MAIN						-
//-----------------------------------------------
int main()
{
	DWORD Temps_Ecoule;	//permet de synchroniser les FPS (images par seconde)
	
	// Open our window/screen
    if (!ptc_open("VrS! Stars",WIDTH,HEIGHT)) return 1;

#ifdef MINIFMOD
	// init FMOD
	FSOUND_File_SetCallbacks(memopen, memclose, memread, memseek, memtell);
	pModule = FMUSIC_LoadSong( (char*)music, NULL ); //sampleloadcallback;

	if (!pModule)
	{
		MessageBox(NULL,"module not found","FMOD error",MB_OK);
		return;		// Quit the program
	}
	FMUSIC_PlaySong(pModule); // start music	
#endif
#ifdef AHX
	tune_init();
/*	player = new AHXPlayer;
	player->Init(NULL);
	player->LoadSong(music, music_length);
	player->InitSubsong(0);
*/
#endif

	initfx();

	// main loop exit
    while (1)
    {
		//on lance l'affichage en synchronisant les FPS
		if (timeGetTime()-Temps_Ecoule>(1000/60))	//60 fps (1000 ms /16.6 ms)
		{
			Temps_Ecoule=timeGetTime();
			render_effect();
#ifdef AHX
			//tune_play();
#endif
		}
    }
}

//-----------------------------------------
void initfx()
{
	#ifdef STARS
	InitHStars();
	#endif
	#ifdef STARSP1
	initStarsP1();
	#endif
	#ifdef STARSP3
	initStarsP3();
	#endif
#ifdef TUNNEL
	prepal();
#endif
#ifdef RASTERL
	initRasterL();
#endif
#ifdef RASTERROLL2
	make_colours();
#endif
}

void render_effect()
{
		Cls(0); // clear screen
	#ifdef STARS
		HStars();
	#endif
	#ifdef STARSP1
		DoStarsP1();
	#endif
	#ifdef STARSP3
		DoStarsP3();
	#endif
#ifdef TUNNEL
		DoDotsTunnel();
#endif
#ifdef RASTERL
		DoRasterL();
#endif
#ifdef RASTERROLL
		DoRasterRoll();
#endif
#ifdef RASTERROLL2
		do_twisty();
#endif
//		dbftext2(340,240,17,RGB(255,255,255));
		writeletter(320,240,1,RGB(255,255,255));
	
		dycpscroll();

        ptc_update(buffer); // update screen with DDraw
}
//--------------------------------------------
// Common Functions
//--------------------------------------------
void stopzic()
{
#ifdef MINIFMOD
	FMUSIC_StopSong(pModule);
	FMUSIC_FreeSong(pModule);
#endif
#ifdef AHX
	tune_shutdown();
#endif
}

// clear screen
void Cls(int rgb)
{
	int x;
	int y;
	//Do cls stuff
	for (y=0; y<HEIGHT; y++)
	{
		for (x=0; x<WIDTH; x++)
		{
            buffer[(y*WIDTH+x)] =rgb;
		}
	}
}

// plot a pixel
void WritePixel(int x,int y,int rgb)
{
	//Do write pixel
	if(x>-1)
	{
		if(x<WIDTH)
		{
			if(y>-1)
			{
				if(y<HEIGHT)
				{
					buffer[(y*WIDTH+x)] =rgb;
				}
			}
		}
	}
}

void edge(int x1,int y1,int x2,int y2,int color)
{
            float  xdistance,ydistance;
            int     i,h2;
            float  StartX,StartY,XRatio,YRatio;

            xdistance = x2 - x1;
            ydistance = y2 - y1;

            h2 = sqrt( xdistance * xdistance + ydistance * ydistance );

            StartX = x1;
            StartY = y1;

            XRatio = xdistance * ( 1.0 / h2 );
            YRatio = ydistance * ( 1.0 / h2 );

        for(i = 0; i<h2; i++)
        {
            if(StartX>0 && StartX<WIDTH && StartY>0 && StartY<HEIGHT-1)
            {
                buffer[ (int)StartX + (int)(StartY * WIDTH ) ] = color;
            }
            StartX = StartX + XRatio;
            StartY = StartY + YRatio;
        }
}